module bram_dp
#(
    parameter width = 16,
    parameter depth_log = 10,
    parameter port_a_register = "true",
    parameter port_b_register = "false"
)
(
    //port A
    input                   clock_a,
    input                   enable_a,
    input                   write_enable_a,
    input   [width-1:0]     write_data_a,
    input	[depth_log-1:0]	addr_a,
    output	reg	[width-1:0]	read_data_a,
    //port B
    input                   clock_b,
    input                   enable_b,
    input                   write_enable_b,
    input   [width-1:0]     write_data_b,
    input	[depth_log-1:0]	addr_b,
    output	reg	[width-1:0]	read_data_b
);

//check parameter sanity
initial begin
    if((port_a_register != "true") & (port_a_register != "false")) begin
        $display("Incorrect setting for port_a_register parameter, please set to true/false");
        $finish();
    end
    if((port_b_register != "true") & (port_b_register != "false")) begin
        $display("Incorrect setting for port_b_register parameter, please set to true/false");
        $finish();
    end
end

reg	[width-1:0]	memory[2**depth_log-1:0];
reg	[width-1:0]	port_a_memory_output;
reg	[width-1:0]	port_b_memory_output;	
    
always @(posedge clock_a) 
    if (enable_a) begin
        if (write_enable_a) 
            memory[addr_a] <= write_data_a;
        port_a_memory_output <= memory[addr_a]; 
    end

always @(posedge clock_b) 
    if (enable_b) begin
        if (write_enable_b) 
            memory[addr_b] <= write_data_b;
        port_b_memory_output <= memory[addr_b]; 
    end
    
generate
if(port_a_register == "true") begin: a_reg
    always @(posedge clock_a)
        read_data_a <= port_a_memory_output;
end else if(port_a_register == "false") begin: a_comb
    always @*
        read_data_a = port_a_memory_output;
end
endgenerate

generate
if(port_b_register == "true") begin: b_reg
    always @(posedge clock_b)
        read_data_b <= port_b_memory_output;
end else if(port_b_register == "false") begin: b_comb
    always @*
        read_data_b = port_b_memory_output;
end
endgenerate
    
endmodule
 