module controller(
    input clock,
    input reset,

    output read_plaintext,
    input plaintext_fifo_empty,
    input ciphertext_fifo_full,

    input iteration_done,
    output final_iteration,
    input ciphertext_found,

    output reg write_ciphertext,
    output reg [15:0] ciphertext_value
);

localparam READ_PLAINTEXT = 0,
           DO_MATH = 1;

reg [1:0] state;
reg [1:0] next_state;

reg [15:0] cnt;

wire ciphertext_known;

always @(posedge clock)
    if(reset)
        state <= READ_PLAINTEXT;
    else
        state <= next_state;

always @*
    case(state)
        READ_PLAINTEXT: next_state = (plaintext_fifo_empty | ciphertext_fifo_full) ? READ_PLAINTEXT : DO_MATH;
        DO_MATH: next_state = write_ciphertext ? READ_PLAINTEXT : DO_MATH;
        default: next_state = READ_PLAINTEXT;
    endcase

always @(posedge clock)
    if(reset | state == READ_PLAINTEXT)
        cnt <= 1;
    else if(state == DO_MATH & iteration_done)
        cnt <= cnt + 1;

assign read_plaintext = (state == READ_PLAINTEXT) & (~plaintext_fifo_empty & ~ciphertext_fifo_full);
assign ciphertext_known = iteration_done & (cnt >= 250) & (final_iteration | ciphertext_found);
assign final_iteration = (state == DO_MATH) & (cnt == 65535);

always @(posedge clock) begin
    write_ciphertext <= ciphertext_known;
    ciphertext_value <= cnt;
end

endmodule
